var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var sql_exports = {};
__export(sql_exports, {
  default: () => sql_default
});
module.exports = __toCommonJS(sql_exports);
var import_database = require("@nocobase/database");
const updateCollection = async (ctx, transaction) => {
  const { filterByTk, values } = ctx.action.params;
  const repo = ctx.db.getRepository("collections");
  const collection = await repo.findOne({
    filter: {
      name: filterByTk
    },
    transaction
  });
  const existFields = await collection.getFields({ transaction });
  const deletedFields = existFields.filter((field) => {
    var _a;
    return !((_a = values.fields) == null ? void 0 : _a.find((f) => f.name === field.name));
  });
  for (const field of deletedFields) {
    await field.destroy({ transaction });
  }
  const upRes = await repo.update({
    filterByTk,
    values,
    updateAssociationValues: ["fields"],
    transaction
  });
  return { collection, upRes };
};
var sql_default = {
  name: "sqlCollection",
  actions: {
    execute: async (ctx, next) => {
      let {
        values: { sql }
      } = ctx.action.params;
      sql = sql.trim().split(";").shift();
      if (!sql) {
        ctx.throw(400, ctx.t("SQL is empty"));
      }
      if (!/^select/i.test(sql) && !/^with([\s\S]+)select([\s\S]+)/i.test(sql)) {
        ctx.throw(400, ctx.t("Only select query allowed"));
      }
      const tmpCollection = new import_database.SqlCollection({ name: "tmp", sql }, { database: ctx.db });
      const model = tmpCollection.model;
      const data = await model.findAll({ attributes: ["*"], limit: 5, raw: true });
      let fields = {};
      try {
        fields = model.inferFields();
      } catch (err) {
        ctx.logger.warn(`resource: sql-collection, action: execute, error: ${err}`);
        fields = {};
      }
      const sources = Array.from(new Set(Object.values(fields).map((field) => field.collection)));
      ctx.body = { data, fields, sources };
      await next();
    },
    setFields: async (ctx, next) => {
      const transaction = await ctx.app.db.sequelize.transaction();
      try {
        const {
          upRes: [collection]
        } = await updateCollection(ctx, transaction);
        await collection.loadFields({
          transaction
        });
        await transaction.commit();
      } catch (e) {
        await transaction.rollback();
        throw e;
      }
      await next();
    },
    update: async (ctx, next) => {
      const transaction = await ctx.app.db.sequelize.transaction();
      try {
        const { upRes } = await updateCollection(ctx, transaction);
        const [collection] = upRes;
        await collection.load({ transaction, resetFields: true });
        await transaction.commit();
        ctx.body = upRes;
      } catch (e) {
        await transaction.rollback();
        throw e;
      }
      await next();
    }
  }
};
